local MAJOR, MINOR = "LibAddonMenu-2.0-OOP", 1
local lam, oldminor = LibStub:NewLibrary(MAJOR, MINOR)
if not lam then return end	--the same or newer version of this lib is already loaded into memory 

local lam2 = LibStub("LibAddonMenu-2.0")

setmetatable(lam, { __call = lam.CreateControlPanel })

function lam.CreateControlPanel(controlPanelName, displayName)
    local self = {}
    setmetatable(self, lam.Panel)
	local data = {
		type = "panel",
		name = controlPanelName,
		displayName = displayName
    }
    self.name = controlPanelName
    self.options = {}

	lam2:RegisterAddonPanel(controlPanelName, data)
    lam2:RegisterOptionControls(controlPanelName, self.options)
    return self
end

lam.Panel = {}

function lam.Panel:AddHeader(text)
	local data = {
		type = "header",
		name = text
	}
	table.insert(self.options, data)
    return self
end

function lam.Panel:AddSlider(text, tooltip, minValue, maxValue, step, getFunc, setFunc, warningText)
	local data = {
		type = "slider",
		name = text,
		tooltip = tooltip,
		min = minValue,
		max = maxValue,
		step = step,
		getFunc = getFunc,
		setFunc = setFunc,
		warning = warningText
	}
    table.insert(self.options, data)
    return self
end

function lam.Panel:AddDropdown(text, tooltip, validChoices, getFunc, setFunc, warningText)
	local data = {
		type = "dropdown",
		name = text,
		tooltip = tooltip,
		choices = validChoices,
		getFunc = getFunc,
		setFunc = setFunc,
		warning = warningText
	}
    table.insert(self.options, data)
    return self
end

function lam.Panel:AddCheckbox(text, tooltip, getFunc, setFunc, warningText)
	local data = {
		type = "checkbox",
		name = text,
		tooltip = tooltip,
		getFunc = getFunc,
		setFunc = setFunc,
		warning = warningText
	}
    table.insert(self.options, data)
    return self
end

function lam.Panel:AddColorPicker(text, tooltip, getFunc, setFunc, warningText)
	local data = {
		type = "colorpicker",
		name = text,
		tooltip = tooltip,
		getFunc = getFunc,
		setFunc = setFunc,
		warning = warningText
	}
    table.insert(self.options, data)
    return self
end

function lam.Panel:AddEditBox(text, tooltip, isMultiLine, getFunc, setFunc, warningText)
	local data = {
		type = "editbox",
		name = text,
		tooltip = tooltip,
		getFunc = getFunc,
		setFunc = setFunc,
		isMultiLine = isMultiLine,
		warning = warningText
	}
    table.insert(self.options, data)
    return self
end

function lam.Panel:AddButton(text, tooltip, onClick, warningText)
	local data = {
		type = "button",
		name = text,
		tooltip = tooltip,
		func = onClick,
		warning = warningText
	}
    table.insert(self.options, data)
    return self
end

function lam.Panel:AddDescription(text, titleText)
	local data = {
		type = "description",
		title = titleText,
		text = text
	}
    table.insert(self.options, data)
    return self
end

function lam.Panel:AddSubMenu(text, tooltip)
    if self.subpanel then return nil end

    local sub = {}
    setmetatable(sub, lam.Panel)
    sub.options = {}
    sub.subpanel = true

	local data = {
		type = "submenu",
		name = text,
		tooltip = tooltip,
		controls = sub.options
	}
	table.insert(self.options, data)
	
	return sub
end

function lam.Panel:AddCustom(reference, width, additionalData)
    if additionalData == nil then
        additionalData = {}
    end

    additionalData.type = "custom"
    additionalData.reference = reference
    additionalData.width = width
    table.insert(self.options, additionalData)
    return self
end

function lam.Panel:AddTexture(image, w, h, tooltip, width, reference)
    local data = {
        type = "texture",
        image = image,
        imageWidth = w,
        imageHeight = h,
        tooltip = tooltip,
        width = width,
        reference = reference
    }

    table.insert(self.options, data)
    return self
end